import sys
import math
import os

TP = 1
transitionProbDict = {}
emissionProbDict = {}
obsSeqList = []
tagStateList = []
tagStateDict = {}
outgoingTagCountDict = {}
noOfTag = 0
AllWordsList = {}
# Viterbi


def Viterbi(seq):
    score = 0
    Seq = seq.split()
    T = len(Seq)
    h, w = noOfTag + 1, T
    viterbi = [[0 for x in range(w)] for y in range(h)]
    backtrack = [[0 for x in range(w)] for y in range(h)]
    for s in tagStateDict.keys():

        emiKey = Seq[0] + '|' + tagStateDict[s]
        if Seq[0] not in AllWordsList.keys():
            multPE = 1
        elif emiKey not in emissionProbDict.keys():
            multPE = 0
        else:
            multPE = emissionProbDict[emiKey]
        tranKey = 'Q0-'+tagStateDict[s]
        if tranKey not in transitionProbDict:
            multPT = 1 / (outgoingTagCountDict['Q0'] + noOfTag)
        else:
            multPT = transitionProbDict[tranKey]
        viterbi[s][0] = multPT * multPE
        backtrack[s][0] = 0
    for t in range(1, T):
        for s_to in tagStateDict.keys():
            for s_from in tagStateDict.keys():
                emiKey = Seq[t] + '|' + tagStateDict[s_to]
                if Seq[t] not in AllWordsList.keys():
                    multPE = 1
                elif emiKey not in emissionProbDict.keys():
                    multPE = 0
                else:
                    multPE = emissionProbDict[emiKey]
                tranKey = tagStateDict[s_from] + '-' + tagStateDict[s_to]
                if tranKey not in transitionProbDict:
                    multPT = 1 / \
                        (outgoingTagCountDict[tagStateDict[s_from]] + noOfTag)
                else:
                    multPT = transitionProbDict[tranKey]
                score = viterbi[s_from][t-1] * multPT * multPE
                if score > viterbi[s_to][t]:
                    viterbi[s_to][t] = score
                    backtrack[s_to][t] = s_from
                else:
                    continue
    best = 0
    for i in tagStateDict.keys():
        if viterbi[i][T-1] > viterbi[best][T-1]:
            best = i
    path = [Seq[T-1]+'/'+tagStateDict[best]]
    nice_path = [tagStateDict[best]]
    for t in range(T-1, 0, -1):
        best = backtrack[best][t]
        path[0:0] = [Seq[t-1]+'/'+tagStateDict[best]]
        nice_path[0:0] = [tagStateDict[best], '--%s-->' % (Seq[t - 1],)]
        nice_path_string = ' '.join(nice_path)
    return (path)

model_path = os.path.dirname(__file__)
model_path = model_path + '/storage/hmmmodel.txt'
modelFile = open(model_path, 'r')
iCount = 0
for line in modelFile:
    if iCount == 0:
        iCount = iCount + 1
        noOfTag = int(line.split(':')[1])
        continue
    if iCount == 1:
        # tags
        iCount = iCount + 1
        tagSet = line.split(':')[1]
        tagSet = tagSet.strip('\n')
        tagStateList = tagSet.split(',')
        continue
    if line == 'Outgoing Count:\n':
        TP = 2
        continue
    if line == 'Transition Probability:\n':
        TP = 1
        continue
    if TP == 2:
        d = line.split(':')
        # print(d[0])
        outgoingTagCountDict[d[0]] = int(d[1].strip('\n'))
    if line == 'Emission Probability:\n':
        TP = 0
        continue
    if TP == 1:
        data = line.split(':')
        if len(data[0]) < 4:
            continue
        tp = data[0]  
        tp = tp.replace('Begin', 'Q0')
        transitionProbDict[tp] = float(data[1].strip('\n'))
    if TP == 0:
        data = line.split(':->')
        tagE = data[0]
        tagE = tagE[2:len(tagE) - 1]
        corpusWord = tagE.split('|')[0]
        AllWordsList[corpusWord] = 1
        emissionProbDict[tagE] = float(data[1].strip('\n'))
# read input file
c = 0
for tagNm in tagStateList:
    tagStateDict[c] = tagNm
    c += 1

def pos_tag(sentences):
    inputText = sentences
    line = inputText
    path = Viterbi(line)
    st = '$'
    for i in path:
        st = st + i + ' '
    st = st.strip('$')
    st = st.strip(' ')
    st = st + '\n'

    return st




import nltk
from nltk.tokenize import word_tokenize 
import string
import re
import glob
import os 

global names, kalimat, output, total
cerita=[]
listStop=[]
namadepan=[]
jawaban=[]
output=[]
sanskerta=[]
id_text = 0
total  = 0
punc = '''!()-[]{};:.'"”\<>/?@#$%^&*_~'''

gender=['I','Ni','Bagus','Ayu']
urutankelahiran=['Putu','Gede','Wayan','Luh','Made','Madé','Kadek','Nengah','Nyoman','Komang','Ketut']
wangsa=['Ida','Anak','Cokorda','Tjokorda','Gusti','Dewa','Sang','Ngakan','Bagus','Desak']
singkatan=['IB','IA','Gde','Gd','Cok','AA','Gst','Dw','Ngkn','Dsk.','W','Wy','Wyn','Pt','Ngh','Md','N','Nymn','Ny','Kt','Dayu','Pan','Men']

namadepan.append(gender)
namadepan.append(urutankelahiran)
namadepan.append(wangsa)
namadepan.append(singkatan)
path = os.path.dirname(__file__)
vocab_path = os.path.dirname(__file__) + "/storage/BaliVocab.txt"
with open( vocab_path, 'r' ) as s_file:
    for line in s_file:
        stripped_line = line.strip('\n')
        listStop.append(stripped_line)

sansekerta_path = os.path.dirname(__file__) + "/storage/sansekertavocab.txt"
with open(sansekerta_path,'r',encoding="utf8" ) as s_file:
    for line in s_file:
        stripped_line = line.strip('\n')
        sanskerta.append(stripped_line)

names=[] 
def ner_name(sentences):
    global output
    kalimat = sentences
    kalimat = nltk.tokenize.sent_tokenize(kalimat)

    for sindex,i in enumerate(kalimat):
        for j in i:
            if(j in punc):
                kalimat[sindex] = kalimat[sindex].replace(j, "")
        kalimat[sindex] = nltk.tokenize.word_tokenize(i)
    for sindex, a in enumerate(kalimat):
        for gindex, b in enumerate(a):
            kalimat[sindex][gindex] = re.sub('ne$', '', b)

    for sindex, sentence in enumerate(kalimat):
        for gindex, a in enumerate(sentence):
            if ([b for b in listStop if a == b] or [b for b in listStop if a.lower() == b]) :
                continue
            elif(a in(item for sublist in namadepan for item in sublist)):
                names.append([a])
                temp = names.index([a])
                for c in range((gindex+1),(len(kalimat[sindex]))):
                    try:
                        if(kalimat[sindex][c][0].isupper()):
                            names[temp].append(kalimat[sindex][c])
                        else:
                            break
                    except:
                        continue
                continue
            try:
                if(a[0].isupper()):
                    if([b for b in sanskerta if a.lower() == b] or [b for b in sanskerta if a.lower() == b]):
                        names.append([a])
                        temp = names.index([a])
                        for c in range((gindex+1),(len(kalimat[sindex]))):
                            try:
                                if(kalimat[sindex][c][0].isupper()):
                                    names[temp].append(kalimat[sindex][c])
                                else:
                                    break
                            except:
                                continue
                        continue
            except:
                continue
            else:
                continue

    for i in names:
        if(len(i)>1):
            i = ' '.join(i)
            output.append(i)

    same_name=[]
    output = list(dict.fromkeys(output))
    copy = output
    for i in range(0,len(output)):
        for j in range(0,len(output)):
            if((copy[i] in output[j]) and i!=j):
                same_name.append(copy[i])

    output = [e for e in output if e not in same_name]
    
    name = ', '.join(map(str, output))
    name = "Nama : " + name
    return name                       


# PENTING!
# Penulisan input harus diakhiri dengan tanda baca, n kalo nama lokasi ya pake hurup kapital
# Misal, Desa Karangasem. dimana karangasem-nya pake huruf kapital n diakhiri tanda baca, kalo ga gitu ya ga kedetect
# Inputnya pake perulangan, kalo mau suud tinggal enter jak

import os, string, re, nltk, csv
from nltk.tokenize import word_tokenize
documents = {}
punctuation = ".?!"
conjungtion = ['tur', 'lan', 'miwah', 'sareng', 'utawi']
days = ['Senin', 'Selasa', 'Rabu', 'Kamis', 'Jumat', 'Sabtu', 'Minggu', 'Redite', 'Soma', 'Anggara', 'Buda', 'Wraspati', 'Sukra', 'Caniscara']
month = ['Januari', 'Februari', 'Maret', 'April', 'Mei', 'Juni', 'Juli', 'Agustus', 'September', 'Oktober', 'November', 'Nopember', 'Desember']
notLocation = ['Sang', 'Hyang', 'Ida', 'Sanghyang', 'Dewa', 'Dewi', 'Tuhan', 'Bapak', 'Ibu', 'Rp', 'Raja', 'Ratu', 'Betari', 'PT', 'Basa', 'Legenda', 'Patih', 
'Perguruan', 'Fakultas', 'Universitas', 'Institut', 'TK', 'SD', 'SMP', 'SMA', 'SMK', 'SMEAN', 'ITB', 'Teknik', 'Institut',
'Dewata', 'Kubilai', 'Purwa', 'Calonarang', 'Babad', 'Ramayana', 'Rama', 'Subali', 'Sugriwa', 'Rahwana', 'Sayyidina', 'Puputan']

area = ['wewidangan', 'widang', 'gumi', 'wilayah', 'bongkol', 'jagat', 'sawengkon', 'wawengkon']
prefixsLocation = ['Gunung', 'Danau', 'Danu', 'Kebun', 'Pasih', 'Segara','Peken', 'Desa', 'Kelurahan', 'Kecamatan', 'Kabupaten', 'Kota', 'Provinsi', 
'Propinsi', 'Pulau', 'Pulo', 'Nusa', 'Tanjung', 'Pasisi', 'Pura', 'Rumah', 'Museum', 'Bandara', 'Kompleks', 'Jalan', 'Gedong', 'Setra', 'Taman', 'Candi',
'Kerajaan', 'Krajan', 'Kesultanan', 'Jembatan', 'Pelabuhan', 'Pelabuan', 'Pesisi']
prepotitionPrefixs = ['ring', 'di', 'saking', 'ka', 'ke', 'uli', 'Ring', 'Di', 'Saking', 'Ka', 'Ke', 'Uli']
directionssuffix = ['Utara', 'Timur', 'Selatan', 'Barat', 'Kaja', 'Kangin', 'Kelod', 'Kauh', 'Tenggara', 'Tengah', 'Loka', 'Kulon']
directionprefix = ['arep', 'batan', 'badauh', 'tengahing', 'ajeng', 'madianing', 'pusat', 'ungkur', 'muncuk']


def listDocuments():
    dokumenList = os.listdir('documents')
    global documents
    for i in dokumenList:
        index = re.findall("(\\d+)", i)[0]
        documents[index] = i
    documents_sorted = {k: v for k, v in sorted(documents.items(), key=lambda item: item[0])}
    documents = documents_sorted
    dataPreprocessing = readDocuments(documents)
    return dataPreprocessing

def readDocuments(documents):
    dataPreprocessing = {}
    for index in documents:
        text = open('documents/'+documents[index])
        text = text.read()
        tittle = text.partition('\n')[0]
        text = text.replace(tittle, "", 1)
        prep = preprocessing(text)
        dataPreprocessing[index] = prep
    return dataPreprocessing

def preprocessing(text):
    symbols = "\"#$%&*+:;<=>@[\]^_`{|}~\n"
    text = text.replace('é','e')
    text = text.replace('Ã©','e')
    text = text.replace('\n',' ')
    text = text.translate(str.maketrans("","", symbols))
    tokens = nltk.tokenize.word_tokenize(text)
    return tokens

def ruleBased(textPrep, locations):
    for index, i in enumerate(textPrep):
        # aturan tanggal
        date = []
        for j in range(len(i)):
            for k in range(j, len(i)-1):
                if i[k].isdigit() and i[k+1] == '/' and i[k+2].isdigit():
                    date.append(i)
                else:
                    break
        if i in date:
            if textPrep[index-1].istitle():
                locations.append(textPrep[index-1])
        
        # aturan tempat, tanggal 
        if i == ',' and textPrep[index+1].isdigit() and textPrep[index+2] in month: # benerin Wanita Batu (drama/monolog,2006)
            temp = []
            for j in range(index-1, -1, -1): 
                if textPrep[j] and textPrep[j] not in prepotitionPrefixs and textPrep[j] not in prefixsLocation and textPrep[j] not in notLocation:
                    if textPrep[j].istitle() or textPrep[j].isupper() and not textPrep[j].isdigit() and textPrep[j] not in punctuation:
                        temp.append(textPrep[j])
                    else:
                        break
                else:
                    temp = []
                    break
            if(temp):
                locations.append(" ".join(reversed(temp)))
        
        # aturan mata angin
        if i in directionssuffix: #benerin NUSA TENGGARA BARAT
            temp = []
            lenght = 0
            if textPrep[index+1] not in directionssuffix: 
                if textPrep[index-1] is not None and textPrep[index-1] not in prepotitionPrefixs and textPrep[index+1] not in directionssuffix and textPrep[index-1] not in prefixsLocation and (textPrep[index-1].istitle() or textPrep[index-1].isupper()):
                    temp.append(i)
                    temp.append(textPrep[index-1])
                    for j in range(index-2, -1, -1):
                        if textPrep[j] and textPrep[j] not in prepotitionPrefixs and textPrep[j] not in prefixsLocation and textPrep[j] not in punctuation and not textPrep[j].isdigit():   
                            if textPrep[j].istitle() or textPrep[j].isupper():
                                temp.append(textPrep[j])
                            else:
                                break
                        else:
                            temp = []
                            break
                else:
                    temp = []
            for j in temp:
                if j in directionssuffix:
                    lenght += 1
            if lenght == len(temp):
                temp = []
            if(temp):                
                locations.append(" ".join(reversed(temp)))        
                
        # aturan preprosisi ('ring', 'di', 'saking', 'ka', 'uli')
        if i in prepotitionPrefixs:
            temp = []
            if textPrep[index+1] not in directionssuffix and textPrep[index+1] not in notLocation and not textPrep[index+1].isdigit():
                for j in range(index+1, len(textPrep)-1):
                    if textPrep[j] not in punctuation and (textPrep[j].istitle() or textPrep[j].isupper() or textPrep[j].isdigit()):
                        temp.append(textPrep[j])
                    elif textPrep[j] == 'de':
                        temp.append(textPrep[j])
                    else:
                        if textPrep[j] == 'diri':
                            del temp[-1]
                        break
            if(temp):          
                locations.append(" ".join(temp))
                
        # aturan pake kata depan lokasi
        if i in prefixsLocation:
            temp = []
            if i is textPrep[0]: #Gunung Agung
                if textPrep[index+1] not in punctuation and (textPrep[index+1].istitle() or textPrep[index+1].isdigit()): #and not textPrep[index+1].isdigit()
                    temp.append(i)
            else: #blablabla Gunung Agung
                if textPrep[index-1] not in notLocation and textPrep[index-1] not in prepotitionPrefixs and textPrep[index-1] not in prefixsLocation and (textPrep[index+1].istitle() or textPrep[index+1].isdigit()):
                    temp.append(i)
                else:
                    temp = []
                    continue
            for j in range(index+1, len(textPrep)-1):
                if textPrep[j] not in punctuation and (textPrep[j].istitle() or textPrep[j].isdigit()):
                    temp.append(textPrep[j])
                else:
                    break

            if(temp):
                locations.append(" ".join(temp))

        # aturan pake area, misalnya wewidangan Karangasem
        if i in area:
            temp = []
            for j in range(index+1, len(textPrep)-1):
                if textPrep[j] not in punctuation and textPrep[j].istitle():
                    if textPrep[j] not in directionssuffix and textPrep[j] not in prefixsLocation and textPrep[j] not in prepotitionPrefixs:
                        temp.append(textPrep[j])
                    else:
                        temp = []
                        break
                else:
                    break
            if(temp):          
                locations.append(" ".join(temp))

        # aturan conjongtion, aturan untuk lokasi beruntun yang langsung nama, kek Desa Pertima, Karangasem
        if i in conjungtion or i == ',':
            temp = []
            loc = []
            locjoin = []
            for j in range(index-1, -1, -1): 
                if textPrep[j].istitle():
                    loc.append(textPrep[j])
                else:
                    break
            if loc:
                locjoin.append(" ".join(reversed(loc)))
            if locjoin and locjoin[0] in locations:
                for j in range(index+1, len(textPrep)-1):
                    if textPrep[j] not in punctuation and (textPrep[j].istitle() or textPrep[j].isupper()):
                        if textPrep[j] not in directionssuffix and textPrep[j] not in prefixsLocation and textPrep[j] not in prepotitionPrefixs and textPrep[j] not in notLocation:
                            temp.append(textPrep[j])
                        else:
                            temp = []
                            break
                    else:
                        break
            if(temp):          
                locations.append(" ".join(temp))

    return locations

def showLocations(locationRetrieve):
    for key, val in locationRetrieve.items():
        locations = ", ".join(val)
        print("Location(s) in document-"+key+": ", end ="")
        if(locations):
            print(locations, end ="")
        else:
            print("-", end ="")
        print('\n', end ="")
    print('\n', end ="")    

def writeToFile(locationRetrieve):
    with open('evaluation/locationRetrieve.txt', 'w', newline='') as dataLocation:
        csvWriter = csv.writer(dataLocation, delimiter=':')
        for key, val in locationRetrieve.items():
            locations = ", ".join(val)
            csvWriter.writerow([key, locations])
            
def ner_location(sentences):
    text = sentences
    locations = []
    textPrep = preprocessing(text)
    locations = ruleBased(textPrep, locations)
    same_place=[]
    locations = list(dict.fromkeys(locations))
    copy = locations
    for i in range(len(locations)-1,0,-1):
        for j in range(len(locations)-1,0,-1):
            if((copy[i] in locations[j]) and i!=j):
                same_place.append(copy[i])
    locations = [e for e in locations if e not in same_place]
    locations = ", ".join(locations)
    locations = "Location : " + locations
    return locations




from setuptools import setup, find_packages
 
classifiers = [
  'Development Status :: 5 - Production/Stable',
  'Intended Audience :: Education',
  'Operating System :: Microsoft :: Windows :: Windows 10',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3'
]
 
setup(
  name='balinese_library',
  version='0.0.7',
  description='A very basic balinese library about information extraction in balinese language',
  long_description=open('README.txt').read() + '\n\n' + open('CHANGELOG.txt').read(),
  url='',  
  packages=['balinese_library'],
  package_data={'': ['*.txt', 'storage/*.txt']},
  author='Ngurah Agus Sanjaya ER',
  author_email='agus_sanjaya@unud.ac.id',
  license='MIT', 
  classifiers=classifiers,
  keywords='Information Extraction, Balinese', 
  install_requires=['nltk'] 
)


import glob
import os
import codecs
import nltk
from nltk.tokenize import word_tokenize
import numpy as np
import re

stop = ['sane', 'ipun', 'ane']
day = ['senin', 'selasa', 'rabu', 'kamis', 'jumat', 'sabtu', 'minggu', 'radite','redite','soma', 'anggara', 'buda', 'respati', 'wraspati', 'wrehaspati', 'sukra', 'saniscara']
month = ['januari', 'februari', 'maret', 'april', 'mei', 'juni','juli', 'agustus', 'september', 'oktober', 'november', 'desember']
date_prefix = ['tahun', 'abad','sasih','bulan','pinanggal', 'dugas', 'dina', 'rahina', 'warsa','duk', 'ngawit', 'periode', 'tanggal', 'sue', 'suene', 'suennyane']
date_sufix = ['lintang', 'mangkin', 'iriki','kaon', 'riin', 'liwat', 'inuni', 'rauh']
date_range = ['nyantos','kantos']
numStr = ['sik','dua', 'a', 'i','kalih', 'tigang', 'telung', 'telu', 'papat','patpat', 'lima', 'nem','nenem', 'pitu', 'kutus', 'sia', 'dasa', 'solas', 'limolas']
duration = ['detik', 'jam', 'dina', 'minggu','menit', 'tiban', 'warsa', 'sasih', 'lemeng']
durationStr = ['abulan', 'telun', 'atiban', 'awai']
notDatePrefix = ['matuuh', 'mayusa', 'no', 'nomer', 'pergub', 'perda']


# s
def preprocessing(text):
    text = text.replace('é', 'e')
    text = text.lower()
    text = nltk.tokenize.word_tokenize(text)
    symbols = "',!\"#$%&()*+.:;<=>?@[\]^_`{|}~\n"
    for i in symbols:
        text = np.char.replace(text, i, '')
        kalimatt = []
    rKalimat = []
    for i in text:
        if i in stop:
            continue
        else:
            rKalimat.append(i)
    text = rKalimat
    for elem in text:
        if elem:
            kalimatt.append(elem)
    # print(kalimatt)
    return kalimatt

def rule(txtPrep, pWaktu):
    for index, i in enumerate(txtPrep):
        # format tanggal Hari 28/08/2019 atau Hari 28-9-2019 atau tgl tok
        if '/' in i or '-' in i:
            m = re. match(
                '(\d{1}|\d{2})[/-](\d{1}|\d{2})[/-](\d{4})', txtPrep[index])
            n = re. match('(\d{1}|\d{2})[/](\d{1}|\d{2})', txtPrep[index])
            # print(m)
            if m or n:
                x = txtPrep[index-1]
                if (x in day):
                    a = x + ' ' + i
                    pWaktu.append(a)
                if txtPrep[index-2] in day:
                    a = x + ' ' + txtPrep[index-2]
                    pWaktu.append(a)
                if x not in day and txtPrep[index-2] not in day and x not in notDatePrefix and txtPrep[index-2] not in notDatePrefix :
                    if m:
                        pWaktu.append(m)
                    if n: 
                        pWaktu.append(n)
            else:
                continue

        # format rentang dengan kata hubung ex: nyantos
        if i in date_range:
            x = txtPrep[index-1]
            y = txtPrep[index+1]
            z = txtPrep[index+2]
            # ex: 60-90 tiban
            if (x.isnumeric() == True and y.isnumeric() == True) and z in duration:
                a = x + '-' + y + z
                pWaktu.append(a)
            # ex: 2014-2019
            if (x.isnumeric() == True and y.isnumeric() == True) and z not in duration and z not in month:
                a = x + '-' + y
                pWaktu.append(a)
            # ex: 1 nyantos 12 februari
            if (x.isnumeric() == True and y.isnumeric() == True) and z in month:
                if txtPrep[index+3].isnumeric()==True:
                    a = x + '-' + y + ' ' + z + ' '+txtPrep[index+3]
                    pWaktu.append(a)
                else:
                    a = x + '-' + y + ' ' + z 
                    pWaktu.append(a)
            
            # ex: 6 sasih
            if z in duration:
                a = y + ' ' + z
                pWaktu.append(a)

        # atiban ane liwat
        if i in durationStr:
            if txtPrep[index-1] not in date_prefix or txtPrep[index-1] not in range:
                if txtPrep[index+1] in date_sufix:
                    a = i + ' ' + txtPrep[index+1]
                    pWaktu.append(a)
                if txtPrep[index+2] in date_sufix:
                    a = i + ' ' + txtPrep[index+2]
                    pWaktu.append(a)
                if txtPrep[index+1] not in date_sufix and txtPrep[index+2] not in date_sufix:
                    pWaktu.append(i)
        # limolas menit, mayusa 40 tahun tidak akan diambil
        if i in numStr or i.isnumeric() == True:
            if 'no' in txtPrep[index-1] or 'nomer' in txtPrep[index-1] or 'nomor' in txtPrep[index-1] or txtPrep[index-1] in notDatePrefix :
                continue
            if txtPrep[index-1] not in date_prefix and txtPrep[index-1] not in day and txtPrep[index-1] not in date_range:
                try:
                    if txtPrep[index+1] in month:
                        if txtPrep[index+2].isnumeric() == True:
                            if txtPrep[index+3] in date_sufix:
                                a = i + ' ' + txtPrep[index +1] + ' ' + txtPrep[index+2]+' '+txtPrep[index+3]
                                pWaktu.append(a)
                            else:
                                a = i + ' ' + txtPrep[index +1] + ' ' + txtPrep[index+2]
                                pWaktu.append(a)
                        if txtPrep[index+2].isnumeric() == False:
                            if txtPrep[index+2] in date_sufix:
                                a = i + ' ' + txtPrep[index +1] + ' ' + txtPrep[index+2]
                                pWaktu.append(a)
                            else:
                                a = i + ' ' + txtPrep[index+1]
                                pWaktu.append(a)

                    if txtPrep[index+1] in duration:
                        if txtPrep[index+2] in date_sufix:
                            a = i + ' ' + txtPrep[index+1] + ' ' + txtPrep[index+2]
                            pWaktu.append(a)
                        if txtPrep[index+3] in date_sufix:
                            a = i + ' ' + txtPrep[index+1] + ' ' + txtPrep[index+3]
                            pWaktu.append(a)
                        if txtPrep[index+2] not in date_sufix and txtPrep[index+3] not in date_sufix:
                            a = i + ' ' + txtPrep[index+1]
                            pWaktu.append(a)
                except:
                    continue
        # format warsa 1990, kalau pergub no xx warsa atau tahun xx ga di detect
        if i in date_prefix:
            z = txtPrep[index-2]
            if 'no' in z or 'nomer' in z or 'pergub' in z or 'perda' in z:
                continue
            else:
                try:
                    x = txtPrep[index+1]
                    y = txtPrep[index+2]
                    z = txtPrep[index+3]
                    j = txtPrep[index+4]
                    if x.isnumeric() == True or x in numStr:
                        # ex: duk 1000 warsa sane sampun lintang, 1998 sane lintang, 1998
                        if y in duration:
                            if z in date_sufix:
                                a = x + ' ' + y + ' ' + z
                                pWaktu.append(a)
                            if j in date_sufix:
                                a = x + ' ' + y + ' ' + j
                                pWaktu.append(a)
                            if z not in date_sufix and j not in date_sufix:
                                a = x + ' ' + y
                                pWaktu.append(a)
            
                        if y not in duration and y not in month and y not in date_range:
                            if y in date_sufix:
                                a = x + ' ' + y
                                pWaktu.append(a)
                            if z in date_sufix:
                                a = x + ' ' + z
                                pWaktu.append(a)
                            if j in date_sufix:
                                a = x + ' ' + j
                                pWaktu.append(a)
                            if y not in duration and y not in date_sufix and z not in date_sufix and j not in date_sufix and '-' not in y:
                                pWaktu.append(x)
                    # warsane mangkin
                    if (txtPrep[index-1].isnumeric() == True or txtPrep[index-1] in numStr):
                        continue
                    if (txtPrep[index-1].isnumeric() == False or txtPrep[index-1] not in numStr) and x.isnumeric() == False:
                        if x in date_sufix:
                            a = i + ' ' + x
                            pWaktu.append(a)
                        if y in date_sufix:
                            a = i + ' ' + y
                            pWaktu.append(a)
                    if y.isnumeric() == True or y in numStr:
                        if z in duration:
                            a = y + ' ' + z
                            pWaktu.append(a)
                    if x in day and y.isnumeric() == False and z not in month and '/' not in y:
                        pWaktu.append(x)
                    # ex: duk 1000 warsa sane sampun lintang, biar duk sama warsa ga tabrakan
                    if txtPrep[index-2] in date_prefix or txtPrep[index-1] in date_prefix:
                        continue
                    # 80-an
                    if '-an' in x or 'ke-' in x:
                        if y in date_range:
                            a = x + '-' + z
                            pWaktu.append(a)
                        else:
                            a = i + ' ' + x
                            pWaktu.append(a)
                    # ex: periode 2019-2020
                    if '-' in x:
                        removed = x.replace('-', '')
                        rlen = len(removed)
                        if rlen == 8:
                            pWaktu.append(x)
                    if x.isnumeric() == True:
                        rlen = len(x)
                        if rlen == 4:
                            if '-' in y and z.isnumeric() == True:
                                a = x + y + z
                                pWaktu.append(a)
                except:
                    continue
                m = 0
                words = []
                while m < 7:
                    words.append(txtPrep[index])
                    m = m + 1
                    index = index + 1
                # format hari 10 januari 2019 atau 10 januari 2019 atau 10 januari
                for index, i in enumerate(words):
                    try:
                        if i in month:
                            x = words[index-1]
                            y = words[index+1]
                            z = words[index-2]
                            j = words[index+2]
                            k = words[index+3]
                            if z in day and x.isnumeric() == True and y.isnumeric() == True:
                                a = z + ' ' + x + ' ' + i + ' ' + y
                                pWaktu.append(a)
                            if z not in day and x.isnumeric() == True and y.isnumeric() == True:
                                a = x + ' ' + i + ' ' + y
                                pWaktu.append(a)
                            if x.isnumeric() == True and y.isnumeric() == False and y not in date_range:
                                a = x + ' ' + i
                                pWaktu.append(a)
                            if x.isnumeric() == False and y.isnumeric() == True:
                                a = i + ' ' + y
                                pWaktu.append(a)
                            # 17-18 mei
                            if '-' in x:
                                if y.isnumeric() == True and j in date_sufix:
                                    a = x + ' ' + i + ' ' + y + ' ' + j
                                    pWaktu.append(a)
                                if y.isnumeric() == False and j in date_sufix:
                                    a = x + ' ' + i + ' ' + y + ' ' + j
                                    pWaktu.append(a)
                                if y.isnumeric() == True and k in date_sufix:
                                    a = x + ' ' + i + ' ' + y + ' ' + k
                                    pWaktu.append(a)
                                if y.isnumeric() == False and k in date_sufix:
                                    a = x + ' ' + i + ' ' + k
                                    pWaktu.append(a)
                                if y.isnumeric() == True and j not in date_sufix and k not in date_sufix:
                                    a = x + ' ' + i + ' ' + y
                                    pWaktu.append(a)
                                if y.isnumeric() == False and j not in date_sufix and k not in date_sufix:
                                    a = x + ' ' + i + ' ' + y
                                    pWaktu.append(a)
                            if y in date_range and k in month:
                                if x.isnumeric() == True:
                                    if words[index+4].isnumeric() == True:
                                        a = x + ' ' + i + '-' + j + ' ' + k + ' '+ words[index+4]
                                        pWaktu.append(a)
                                    else:
                                        a = x + ' ' + i + '-' + j + ' ' + k
                                        pWaktu.append(a)
                            if words[index-2] in date_range and words[index-3] in month:
                                continue
                    except:
                        continue
        # yg tidak diawalin prefix
        if i in day and txtPrep[index-1] not in date_prefix:
            if '/' in txtPrep[index+1] or '-' in txtPrep[index+1]:
                continue
            if txtPrep[index+2] in month:
                a = i + ' ' + txtPrep[index+1] + ' ' + txtPrep[index+2] + ' ' + txtPrep[index+3]
                pWaktu.append(a)
            if txtPrep[index-1] not in date_prefix and txtPrep[index+2] not in month:
                pWaktu.append(i)
    return pWaktu
    
def ner_time(sentences):
    string = sentences
    prep = preprocessing(string)
    pWaktu = []
    pWaktu = rule(prep, pWaktu)
    time = ', '.join(map(str, pWaktu))
    time = "Time : " + time 
    return time